@@ -4,22 +4,22 @@ package outboundgroup
 
 import (
 	C "github.com/metacubex/mihomo/constant"
-	"github.com/metacubex/mihomo/constant/provider"
+	P "github.com/metacubex/mihomo/constant/provider"
 )
 
 type ProxyGroup interface {
 	C.ProxyAdapter
 
-	Providers() []provider.ProxyProvider
+	Providers() []P.ProxyProvider
 	Proxies() []C.Proxy
 	Now() string
 }
 
-func (f *Fallback) Providers() []provider.ProxyProvider {
+func (f *Fallback) Providers() []P.ProxyProvider {
 	return f.providers
 }
 
-func (lb *LoadBalance) Providers() []provider.ProxyProvider {
+func (lb *LoadBalance) Providers() []P.ProxyProvider {
 	return lb.providers
 }
 
@@ -35,7 +35,7 @@ func (lb *LoadBalance) Now() string {
 	return ""
 }
 
-func (r *Relay) Providers() []provider.ProxyProvider {
+func (r *Relay) Providers() []P.ProxyProvider {
 	return r.providers
 }
 
@@ -47,7 +47,7 @@ func (r *Relay) Now() string {
 	return ""
 }
 
-func (s *Selector) Providers() []provider.ProxyProvider {
+func (s *Selector) Providers() []P.ProxyProvider {
 	return s.providers
 }
 
@@ -55,7 +55,7 @@ func (s *Selector) Proxies() []C.Proxy {
 	return s.GetProxies(false)
 }
 
-func (u *URLTest) Providers() []provider.ProxyProvider {
+func (u *URLTest) Providers() []P.ProxyProvider {
 	return u.providers
 }
 
